unit RegDBScr;

//---------------------------------------------------------------------------//
//                                                                           //
//   Registration unit for component DBScroll                                //
//                                                                           //
//   Date:    Jan, 15 1998                                                   //
//   Version: 1.1e                                                           //
//   Author:  Jan - Michael Strube                                           //
//   System:  Delphi 2.01 / C++Builder 1.0 / Delphi 3                        //
//                                                                           //
//   http://ourworld.compuserve.com/homepages/praxisservice/                 //
//                                                                           //
//---------------------------------------------------------------------------//

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DsgnIntf, DBScroll, ExtCtrls, Menus, Grids;

type
  TDBScrollConnectionsDlg = class(TForm)
    BtnReConnect: TButton;
    BtnCancel: TButton;
    BtnOK: TButton;
    BtnDisconnect: TButton;
    BevelBottom: TBevel;
    BtnDown: TButton;
    BtnUp: TButton;
    BtnDelete: TButton;
    PopupMenuDB_Fields: TPopupMenu;
    BtnAddModify: TButton;
    StringGridFields: TStringGrid;
    procedure BtnReConnectClick(Sender: TObject);
    procedure BtnDisconnectClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnUpClick(Sender: TObject);
    procedure BtnDownClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure BtnAddModifyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StringGridFieldsMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure StringGridFieldsSelectCell(Sender: TObject; Col,
      Row: Longint; var CanSelect: Boolean);
    procedure StringGridFieldsKeyPress(Sender: TObject; var Key: Char);
    procedure StringGridFieldsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    ResetColumnAttributes: Boolean;
    ResetColumnWidth: Boolean;
  public
    Grid: TCustomDBScroll;
    DB_Fields: TStringList;
    constructor CreateIt(AOwner: TComponent; AGrid: TCustomDBScroll);
    destructor Destroy; override;
    procedure PopupMenuDB_FieldsClick(Sender: TObject);
  end;

  TDBScrollColumnProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation

uses
  TypInfo, CommCtrl, DB;

{$R *.DFM}

//---------------------------------------------------------------------------//
//                              DataField Property                           //
//---------------------------------------------------------------------------//
type
  TDBStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

function TDBStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result:= [paValueList, paSortList, paMultiSelect];
end;

procedure TDBStringProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values:= TStringList.Create;
  try
    GetValueList(Values);
    for I:= 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

type
  TDataFieldProperty = class(TDBStringProperty)
  public
    function GetDataSourcePropName: string; virtual;
    procedure GetValueList(List: TStrings); override;
  end;

function TDataFieldProperty.GetDataSourcePropName: string;
begin
  Result:= 'DataSource';
end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TCustomDBScroll;
  PropInfo: PPropInfo;
  DataSource: TDataSource;
begin
  // DataSource is a property of TDBScroll
  Instance:= TScrollColumn(GetComponent(0)).DBScrollGrid;
  if Instance <> nil then begin
    PropInfo:= TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then begin
      DataSource:= TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
      if (DataSource <> nil) and (DataSource.DataSet <> nil) then
        DataSource.DataSet.GetFieldNames(List);
    end;
  end;
end;

//---------------------------------------------------------------------------//
//                           ScrollColumn Property                           //
//---------------------------------------------------------------------------//
procedure TDBScrollColumnProperty.Edit;
begin
  with TDBScrollConnectionsDlg.CreateIt(Application,
             TScrollColumn(GetOrdValue).Grid) do try
    ShowModal;
  finally
    Free;
  end;
end;

function TDBScrollColumnProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paSubProperties];
end;

//---------------------------------------------------------------------------//
//              ScrollConnections Dialog (Property-Editor)                   //
//---------------------------------------------------------------------------//
constructor TDBScrollConnectionsDlg.CreateIt(AOwner: TComponent; AGrid: TCustomDBScroll);
var
  LokZaehl: Integer;
  NewMenuItem: TMenuItem;
begin
  Grid:= AGrid;
  DB_Fields:= TStringList.Create;
  if (Grid.DataSource <> nil) and (Grid.DataSource.DataSet <> nil) then
    Grid.DataSource.DataSet.GetFieldNames(DB_Fields);
  inherited Create(AOwner);
  for LokZaehl:= 0 to DB_Fields.Count-1 do begin
    NewMenuItem:= TMenuItem.Create(Self);
    NewMenuItem.Caption:= DB_Fields[LokZaehl];
    NewMenuItem.RadioItem:= True;
    NewMenuItem.OnClick:= PopupMenuDB_FieldsClick;
    PopupMenuDB_Fields.Items.Add(NewMenuItem);
  end;
  ResetColumnAttributes:= False;
  ResetColumnWidth:= False;
end;

procedure TDBScrollConnectionsDlg.FormCreate(Sender: TObject);
var
  CounterCols: Integer;
  CounterFields: Integer;
begin
  with StringGridFields do begin
    ColWidths[0]:= 34;
    Cells[0, 0]:= 'No.';
    Cells[1, 0]:= 'Header';
    Cells[2, 0]:= 'DataField1';
    Cells[3, 0]:= 'DataField2';
    Cells[4, 0]:= 'DataField3';
    Cells[5, 0]:= 'DataField4';
    for CounterCols:= 1 to DBScrMaxColumns do begin
      Cells[0, CounterCols]:= Format('%.2d', [CounterCols]);
      Cells[1, CounterCols]:= Grid.ColumnXX[CounterCols].Header;
      for CounterFields:= 1 to DBScrFieldsPerColumn do
        Cells[CounterFields + 1, CounterCols]:=
           Grid.ColumnXX[CounterCols].DataFields[CounterFields];
    end;
  end;
end;

destructor TDBScrollConnectionsDlg.Destroy;
begin
  DB_Fields.Free;
  inherited;
end;

procedure TDBScrollConnectionsDlg.PopupMenuDB_FieldsClick(Sender: TObject);
begin
  if Sender is TMenuItem then begin
    with StringGridFields do begin
      if (Col = 2) then begin
        if (Cells[1, Row] = '') or (Cells[1, Row] = Cells[2, Row]) then
          Cells[1, Row]:= TMenuItem(Sender).Caption;
      end;
      Cells[Col, Row]:= TMenuItem(Sender).Caption;
    end;
  end;
end;

procedure TDBScrollConnectionsDlg.BtnReConnectClick(Sender: TObject);
var
  MaxCols: Integer;
  LokZaehl: Integer;
begin
  ResetColumnAttributes:= True;
  ResetColumnWidth:= True;
  BtnDisconnect.Click;
  with Grid do begin
    if DataSource = nil then begin
      MessageDlg('Property "' + Grid.Name + '.DataSource" has no valid '+
         'value. Please check this property in the Object Inspector.',
         mtWarning, [mbOK], 0);
      ActiveControl:= BtnOK;
      Exit;
    end;
    if DataSource.DataSet = nil then begin
      MessageDlg('Property "'+ Grid.DataSource.Name + '.DataSet" of the '+
         'connetced DataSource component has no valid value. Please check '+
         'this property in the Object Inspector.', mtWarning, [mbOK], 0);
      ActiveControl:= BtnOK;
      Exit;
    end;
    if DB_Fields.Count < StringGridFields.RowCount-1 then
      MaxCols:= DB_Fields.Count
    else
      MaxCols:= StringGridFields.RowCount-1;

    for LokZaehl:= 1 to MaxCols do begin
      StringGridFields.Cells[1, LokZaehl]:= DB_Fields[LokZaehl-1];;
      StringGridFields.Cells[2, LokZaehl]:= DB_Fields[LokZaehl-1];
    end;
  end;
end;

procedure TDBScrollConnectionsDlg.BtnDisconnectClick(Sender: TObject);
var
  LokZaehlCol, LokZaehlRow: Integer;
begin
  with StringGridFields do
    for LokZaehlCol:= 1 to ColCount-1 do
      for LokZaehlRow:= 1 to RowCount-1 do
         Cells[LokZaehlCol, LokZaehlRow]:= '';
end;

procedure TDBScrollConnectionsDlg.BtnUpClick(Sender: TObject);
var
  TempStringList: TStringList;
  CurrentR: Integer;
begin
  CurrentR:= StringGridFields.Row;
  if CurrentR <= 1 then Exit;
  TempStringList:= TStringList.Create;
  try
    with StringGridFields do begin
      TempStringList.Assign(Rows[CurrentR]);
      Rows[CurrentR].Assign(Rows[CurrentR-1]);
      Rows[CurrentR-1].Assign(TempStringList);
    end;
  finally
    TempStringList.Free;
  end;
  StringGridFields.Row:= CurrentR-1;
end;

procedure TDBScrollConnectionsDlg.BtnDownClick(Sender: TObject);
var
  TempStringList: TStringList;
  CurrentR: Integer;
begin
  CurrentR:= StringGridFields.Row;
  if CurrentR >= StringGridFields.RowCount-1 then Exit;
  TempStringList:= TStringList.Create;
  try
    with StringGridFields do begin
      TempStringList.Assign(Rows[CurrentR]);
      Rows[CurrentR].Assign(Rows[CurrentR+1]);
      Rows[CurrentR+1].Assign(TempStringList);
    end;
  finally
    TempStringList.Free;
  end;
  StringGridFields.Row:= CurrentR+1;
end;

procedure TDBScrollConnectionsDlg.BtnAddModifyClick(Sender: TObject);
var
  ListEntryRect: TRect;
begin
  ActiveControl:= StringGridFields;
  with StringGridFields do begin
    ListEntryRect:= CellRect(Col, Row);
    StringGridFieldsMouseDown(StringGridFields, mbRight,
      [ssRight], ListEntryRect.Left, ListEntryRect.Top);
  end;
end;

procedure TDBScrollConnectionsDlg.BtnDeleteClick(Sender: TObject);
begin
  with StringGridFields do Cells[Col, Row]:= '';
end;

procedure TDBScrollConnectionsDlg.BtnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TDBScrollConnectionsDlg.BtnOKClick(Sender: TObject);
var
  LokZaehl: Integer;
  FieldCounter: Integer;
  OldPos: Integer;
  TempColumns: Array[1..DBScrMaxColumns] of TScrollColumn;
  DsgnForm: {$IFDEF VER100} TCustomForm {$ELSE} TForm {$ENDIF};
  ConnectedCols: Integer;
  TempField: TField;
begin
  ConnectedCols:= 0;
  with Grid do begin
    for LokZaehl:= 1 to StringGridFields.RowCount-1 do begin
      OldPos:= StrToInt(StringGridFields.Cells[0, LokZaehl]);
      TempColumns[LokZaehl]:= ColumnXX[OldPos];
      TempColumns[LokZaehl].Header:= StringGridFields.Cells[1, LokZaehl];
      for FieldCounter:= 1 to StringGridFields.ColCount-2 do
        TempColumns[LokZaehl].DataFields[FieldCounter]:=
           StringGridFields.Cells[FieldCounter+1, LokZaehl];
      if TempColumns[LokZaehl].GetFieldCount > 0 then inc(ConnectedCols);

      if ResetColumnAttributes then begin
        if DataLink.DataSet <> nil then begin
          TempField:= DataLink.DataSet.FindField(TempColumns[LokZaehl].DataField1);
          if TempField <> nil then begin
            TempColumns[LokZaehl].Alignment:= TempField.Alignment;
            TempColumns[LokZaehl].ReadOnly:= TempField.ReadOnly
          end else begin
            TempColumns[LokZaehl].Alignment:= taLeftJustify;
            TempColumns[LokZaehl].ReadOnly:= False;
          end;
        end else begin
          TempColumns[LokZaehl].Alignment:= taLeftJustify;
          TempColumns[LokZaehl].ReadOnly:= False;
        end;
        TempColumns[LokZaehl].CanEnter:= True;
        TempColumns[LokZaehl].DefaultDrawing:= True;
        TempColumns[LokZaehl].DisplayLength:= 1000;
        TempColumns[LokZaehl].HeaderAlignment:= taCenter;
        TempColumns[LokZaehl].OwnerDraw:= False;
        TempColumns[LokZaehl].ParentColor:= True;
      end;
    end;

    for LokZaehl:= 1 to DBScrMaxColumns do begin
      ColumnXX[LokZaehl]:= TempColumns[LokZaehl];
      if ResetColumnWidth then begin
        if ColumnXX[LokZaehl].DataField = '' then
          ColumnXX[LokZaehl].SetWidthPercent2(0)
        else
          if ConnectedCols > 0 then
            ColumnXX[LokZaehl].SetWidthPercent2(100 / ConnectedCols);
      end;
    end;
    CalcRectArea;
    if Grid.HandleAllocated then InvalidateRect(Grid.Handle, nil, False);
  end;
  DsgnForm:= GetParentForm(Grid);
  if DsgnForm <> nil then DsgnForm.Designer.Modified;
  Close;
end;

procedure TDBScrollConnectionsDlg.StringGridFieldsMouseDown(
  Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var
  CurrentSelRect: TRect;
  PopupAt: TPoint;
  SelectX, SelectY: LongInt;
  InMenuPos: Integer;
  LokZaehl: Integer;
begin
  if Button= mbRight then begin
    with StringGridFields do begin
      MouseToCell(X, Y, SelectX, SelectY);
      Col:= SelectX;
      Row:= SelectY;
      CurrentSelRect:= CellRect(SelectX, SelectY);
    end;
    if SelectX = 1 then
      // Header right-clicked
      StringGridFields.EditorMode:= True
    else begin
      // Datafield right-clicked
      PopupAt:= StringGridFields.ClientToScreen(Point(CurrentSelRect.Right,
         CurrentSelRect.Bottom));

      for LokZaehl:= 0 to PopupMenuDB_Fields.Items.Count-1 do
        PopupMenuDB_Fields.Items[LokZaehl].Checked:= False;
      InMenuPos:= DB_Fields.IndexOf(StringGridFields.Cells[SelectX, SelectY]);
      if InMenuPos >= 0 then
        PopupMenuDB_Fields.Items[InMenuPos].Checked:= True;

      PopupMenuDB_Fields.Popup(PopupAt.X - 8, PopupAt.Y - 8);
    end;
  end;
end;

procedure TDBScrollConnectionsDlg.StringGridFieldsSelectCell(
  Sender: TObject; Col, Row: Longint; var CanSelect: Boolean);
begin
  if Col = 1 then
    StringGridFields.Options:= StringGridFields.Options + [goEditing]
  else
    StringGridFields.Options:= StringGridFields.Options - [goEditing];
end;

procedure TDBScrollConnectionsDlg.StringGridFieldsKeyPress(Sender: TObject;
  var Key: Char);
begin
  Case Key Of
    chr(VK_RETURN): begin
      BtnAddModify.Click;
      Key:= #0;
    end;
  End;
end;

procedure TDBScrollConnectionsDlg.StringGridFieldsKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  Case Key Of
    VK_DELETE: BtnDelete.Click;
  End;
end;

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(String), TScrollColumn, 'DataField1',
     TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(String), TScrollColumn, 'DataField2',
     TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(String), TScrollColumn, 'DataField3',
     TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(String), TScrollColumn, 'DataField4',
     TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(TScrollColumn), nil, '',
     TDBScrollColumnProperty);
  RegisterComponents('PrxService', [TDBScroll]);
end;

end.
